@@ -1,4 +1,5 @@
 import asyncio
+import glob
 import hashlib
 import json
 import logging
@@ -20,12 +21,14 @@ from vectorcode.cli_utils import (
     GLOBAL_EXCLUDE_SPEC,
     GLOBAL_INCLUDE_SPEC,
     Config,
+    SpecResolver,
     expand_globs,
     expand_path,
 )
 from vectorcode.common import (
-    get_client,
+    ClientManager,
     get_collection,
+    get_embedding_function,
     list_collection_files,
     verify_ef,
 )
@@ -90,6 +93,7 @@ async def chunked_add(
     max_batch_size: int,
     semaphore: asyncio.Semaphore,
 ):
+    embedding_function = get_embedding_function(configs)
     full_path_str = str(expand_path(str(file_path), True))
     orig_sha256 = None
     new_sha256 = hash_file(full_path_str)
@@ -142,9 +146,21 @@ async def chunked_add(
             async with collection_lock:
                 for idx in range(0, len(chunks), max_batch_size):
                     inserted_chunks = chunks[idx : idx + max_batch_size]
+                    embeddings = embedding_function(
+                        list(str(c) for c in inserted_chunks)
+                    )
+                    if (
+                        isinstance(configs.embedding_dims, int)
+                        and configs.embedding_dims > 0
+                    ):
+                        logger.debug(
+                            f"Truncating embeddings to {configs.embedding_dims} dimensions."
+                        )
+                        embeddings = [e[: configs.embedding_dims] for e in embeddings]
                     await collection.add(
                         ids=[get_uuid() for _ in inserted_chunks],
                         documents=[str(i) for i in inserted_chunks],
+                        embeddings=embeddings,
                         metadatas=metas,
                     )
     except (UnicodeDecodeError, UnicodeError):  # pragma: nocover
@@ -187,22 +203,13 @@ def show_stats(configs: Config, stats: VectoriseStats):
 
 
 def exclude_paths_by_spec(
-    paths: Iterable[str], specs: pathspec.PathSpec | str
+    paths: Iterable[str], spec_path: str, project_root: Optional[str] = None
 ) -> list[str]:
     """
     Files matched by the specs will be excluded.
     """
-    if isinstance(specs, str):
-        with open(specs) as fin:
-            specs = pathspec.GitIgnoreSpec.from_lines(fin.readlines())
-    return [path for path in paths if not specs.match_file(path)]
 
-
-def include_paths_by_spec(paths: Iterable[str], specs: pathspec.PathSpec) -> list[str]:
-    """
-    Only include paths matched by the specs.
-    """
-    return [path for path in paths if specs.match_file(path)]
+    return list(SpecResolver.from_path(spec_path, project_root).match(paths, True))
 
 
 def load_files_from_include(project_root: str) -> list[str]:
@@ -235,10 +242,16 @@ def find_exclude_specs(configs: Config) -> list[str]:
     Load a list of paths to exclude specs.
     Can be `.gitignore` or local/global `vectorcode.exclude`
     """
-    gitignore_path = os.path.join(str(configs.project_root), ".gitignore")
-    specs = [
-        gitignore_path,
-    ]
+    if configs.recursive:
+        specs = glob.glob(
+            os.path.join(str(configs.project_root), "**", ".gitignore"), recursive=True
+        ) + glob.glob(
+            os.path.join(str(configs.project_root), "**", "vectorcode.exclude"),
+            recursive=True,
+        )
+    else:
+        specs = [os.path.join(str(configs.project_root), ".gitignore")]
+
     exclude_spec_path = os.path.join(
         str(configs.project_root), ".vectorcode", "vectorcode.exclude"
     )
@@ -246,69 +259,74 @@ def find_exclude_specs(configs: Config) -> list[str]:
         specs.append(exclude_spec_path)
     elif os.path.isfile(GLOBAL_EXCLUDE_SPEC):
         specs.append(GLOBAL_EXCLUDE_SPEC)
+    specs = [i for i in specs if os.path.isfile(i)]
+    logger.debug(f"Loaded exclude specs: {specs}")
     return specs
 
 
 async def vectorise(configs: Config) -> int:
     assert configs.project_root is not None
-    client = await get_client(configs)
-    try:
-        collection = await get_collection(client, configs, True)
-    except IndexError as e:
-        print(
-            f"{e.__class__.__name__}: Failed to get/create the collection. Please check your config."
-        )
-        return 1
-    if not verify_ef(collection, configs):
-        return 1
-
-    files = await expand_globs(
-        configs.files or load_files_from_include(str(configs.project_root)),
-        recursive=configs.recursive,
-        include_hidden=configs.include_hidden,
-    )
-
-    if not configs.force:
-        for spec_path in find_exclude_specs(configs):
-            if os.path.isfile(spec_path):
-                logger.info(f"Loading ignore specs from {spec_path}.")
-                files = exclude_paths_by_spec((str(i) for i in files), spec_path)
-    else:  # pragma: nocover
-        logger.info("Ignoring exclude specs.")
-
-    stats = VectoriseStats()
-    collection_lock = Lock()
-    stats_lock = Lock()
-    max_batch_size = await client.get_max_batch_size()
-    semaphore = asyncio.Semaphore(os.cpu_count() or 1)
-
-    with tqdm.tqdm(
-        total=len(files), desc="Vectorising files...", disable=configs.pipe
-    ) as bar:
+    async with ClientManager().get_client(configs) as client:
         try:
-            tasks = [
-                asyncio.create_task(
-                    chunked_add(
-                        str(file),
-                        collection,
-                        collection_lock,
-                        stats,
-                        stats_lock,
-                        configs,
-                        max_batch_size,
-                        semaphore,
-                    )
-                )
-                for file in files
-            ]
-            for task in asyncio.as_completed(tasks):
-                await task
-                bar.update(1)
-        except asyncio.CancelledError:
-            print("Abort.", file=sys.stderr)
+            collection = await get_collection(client, configs, True)
+        except IndexError as e:
+            print(
+                f"{e.__class__.__name__}: Failed to get/create the collection. Please check your config."
+            )
+            return 1
+        if not verify_ef(collection, configs):
             return 1
 
-    await remove_orphanes(collection, collection_lock, stats, stats_lock)
+        files = await expand_globs(
+            configs.files or load_files_from_include(str(configs.project_root)),
+            recursive=configs.recursive,
+            include_hidden=configs.include_hidden,
+        )
 
-    show_stats(configs=configs, stats=stats)
-    return 0
+        if not configs.force:
+            for spec_path in find_exclude_specs(configs):
+                if os.path.isfile(spec_path):
+                    logger.info(f"Loading ignore specs from {spec_path}.")
+                    files = exclude_paths_by_spec(
+                        (str(i) for i in files), spec_path, str(configs.project_root)
+                    )
+                    logger.debug(f"Files after excluding: {files}")
+        else:  # pragma: nocover
+            logger.info("Ignoring exclude specs.")
+
+        stats = VectoriseStats()
+        collection_lock = Lock()
+        stats_lock = Lock()
+        max_batch_size = await client.get_max_batch_size()
+        semaphore = asyncio.Semaphore(os.cpu_count() or 1)
+
+        with tqdm.tqdm(
+            total=len(files), desc="Vectorising files...", disable=configs.pipe
+        ) as bar:
+            try:
+                tasks = [
+                    asyncio.create_task(
+                        chunked_add(
+                            str(file),
+                            collection,
+                            collection_lock,
+                            stats,
+                            stats_lock,
+                            configs,
+                            max_batch_size,
+                            semaphore,
+                        )
+                    )
+                    for file in files
+                ]
+                for task in asyncio.as_completed(tasks):
+                    await task
+                    bar.update(1)
+            except asyncio.CancelledError:
+                print("Abort.", file=sys.stderr)
+                return 1
+
+        await remove_orphanes(collection, collection_lock, stats, stats_lock)
+
+        show_stats(configs=configs, stats=stats)
+        return 0
